@@ -20,7 +20,7 @@ import datetime
 import time
 import math
 
-PAST_VERSION="0.1.1"
+PAST_VERSION="0.1.2"
 
 class Resource:
     def __init__( self, name, avail = 100 ):
@@ -39,11 +39,18 @@ class Task:
         self.__res = res
         self.__effort = effort
         self.__completed = completed
-        self.__depends = depends
         self.__prio = prio
         self.__finish_date = None
         self.__actual_prio = 0
 
+        if isinstance( depends, str):
+            self.__fixed_start_date = datetime.date.fromtimestamp( time.mktime( time.strptime( depends,
+                                                                                        "%Y-%m-%d" ) ) )
+            self.__depends = None
+        else:
+            self.__depends = depends
+            self.__fixed_start_date = None
+
     def getName( self ):
         return self.__name
 
@@ -74,6 +81,9 @@ class Task:
     def getActualPrio( self ):
         return self.__actual_prio
 
+    def getFixedStartDate( self ):
+        return self.__fixed_start_date
+
 class Milestone:
     def __init__( self, name, date, buf ):
         self.__name = name
@@ -180,12 +190,19 @@ class PAST:
         if not_earlier is not None:
             t = not_earlier
 
+        if task.getFixedStartDate() is not None:
+            t = task.getFixedStartDate()
+
         while t.weekday() >= 5:
             t += datetime.timedelta( 1 )
         
         if duration > 0:
             while duration > 0:
                 while t in self.__schedule[res]:
+
+                    if task.getFixedStartDate() is not None and self.__schedule[res][t] != task:
+                        raise RuntimeError( "unable to place fixed task", task.getName() )
+
                     t += datetime.timedelta( 1 )
 
                     while t.weekday() >= 5:
@@ -294,15 +311,22 @@ class PAST:
 
         prio = max_prio
 
+        placement_stage = 1  # 1 for fixed tasks, 2 for the rest
+
         while prio >= min_prio:
             for t in milestone.getTasks():
-                if t.getActualPrio() == prio:
+                if ( placement_stage == 1 and t.getFixedStartDate() is not None ) or \
+                   ( placement_stage == 2 and t.getActualPrio() == prio and t.getFixedStartDate() is None ):
                     self.__scheduleTask( t )
 
                     if t.getPriority() >= 0:
                         total_progress += t.getEffort() * t.getCompleted() / 100.0
                         total_progress_count += t.getEffort()
 
+            if placement_stage == 1:
+                placement_stage = 2
+                continue
+
             prio -= 1
 
         if total_progress_count > 0:
@@ -426,6 +450,18 @@ class PAST:
             # no milestone so alway met
             return ( None, 0 )
 
+    def __findMilestone( self, current_time ):
+        hit = None
+
+        for m in self.__milestones:
+            if current_time <= m.getDate():
+                if hit is None:
+                    hit = m
+                elif m.getDate() < hit.getDate():
+                    hit = m
+
+        return hit
+
     def printSchedule( self, name = "plan.html" ):
         fh = open( name, "w" )
 
@@ -437,6 +473,7 @@ class PAST:
         """
 
         print >> fh, """
+        <h1>Milestone overview</h1>
         <table border=\"1\">
         """
 
@@ -479,7 +516,7 @@ class PAST:
                 while current_time.weekday() >= 5:
                     current_time += datetime.timedelta( 1 )
 
-        print >> fh, "<table border=\"1\"><tr><th>Calender week</th>"
+        print >> fh, "<h1>Schedule</h1><table border=\"1\"><tr><th>Calender week</th>"
 
         current_time = smallest
         while current_time <= highest:
@@ -589,7 +626,71 @@ class PAST:
             print >> fh, "</tr>"
 
         print >> fh, """
-        </table>
+        </table>"""
+
+        days_per_milestone = {}
+        milestone_index = 0
+        for m in self.__milestones:
+            days_per_milestone[m] = 0
+
+        used_days_per_milestones = {}
+        for res in self.getResources():
+            used_days_per_milestones[res] = {}
+            for m in self.__milestones:
+                used_days_per_milestones[res][m] = 0
+
+        current_time = smallest
+        while current_time <= highest:
+            while current_time.weekday() >= 5:
+                current_time += datetime.timedelta( 1 )
+
+            while current_time > self.__milestones[milestone_index].getDate():
+                milestone_index += 1
+
+            if milestone_index >= len( self.__milestones ):
+                break
+
+            days_per_milestone[ self.__milestones[milestone_index] ] += 1
+
+            current_time += datetime.timedelta( 1 )
+
+        for res in self.getResources():
+            for current_time in sorted( table[res].keys() ):
+                if current_time.weekday() >= 5:
+                    continue
+
+                task = table[res][current_time]
+
+                if task:
+                    if self.__isVacationTask( task ):
+                        task_milestone = self.__findMilestone( current_time )
+                        if task_milestone:
+                            used_days_per_milestones[res][ task_milestone ] += 1
+                    else:
+                        if task.getPriority() >= 0:
+                            ( task_milestone, met ) = self.__checkMetMilestone( task, self.__milestones )
+                            used_days_per_milestones[res][ task_milestone ] += 1
+
+        print >> fh, """
+        <h1>Workload</h1>
+        <table border=\"1\"><tr><th></th>"""
+        
+        for m in self.__milestones:
+            if days_per_milestone[m] >0 :
+                print >> fh, "<th>", m.getName(), "</th>"
+
+        print >> fh, "</tr>"
+
+        for res in self.getResources():
+            print >> fh, "<tr><td>", res.getName(), "</td>"
+            for m in self.__milestones:
+                if days_per_milestone[m] >0 :
+                    print >> fh, "<td>%d %%</td>" % ( used_days_per_milestones[res][m] * 100 / days_per_milestone[m] )
+            print >> fh, "</tr>"
+
+        print >> fh, "</table>"
+
+        print >> fh, """
         </body>
         </html>
         """
@@ -599,14 +700,14 @@ if __name__ == "__main__":
     d1 = Resource( "dev1", 30 )
     d2 = Resource( "dev2", 100 )
 
-    m1 = Milestone( "v1", date( 2013, 9, 9 ), 5 )
+    m1 = Milestone( "v1", "2013-09-09", 5 )
     m1.add( Task( "t1", d1, 5, 0 ) )
     m1.add( Task( "t2", d2, 15, 25 ) )
     m1.add( Task( "t3", d2, 5, 0 ) )
     m1.add( Task( "t4", d2, 5, 0 ) )
 
-    s = PAST( date( 2013, 7, 29 ) )
-    s.scheduleVacation( d2, date( 2013, 7, 30 ), 5 )
+    s = PAST( "2013-07-29" )
+    s.scheduleVacation( d2, "2013-07-30", 5 )
 
     s.scheduleMilestone( m1 )
 
